<?php
// $Id: 6ga-cancel.php,v 1.14 2012/05/09 16:26:47 name Exp $
//
// 6ga-cancel.php
//
//	Copyright (c) 2005 Hiro Sugawara
//	Licensed under terms of GPL 2.0
//
//	Script to cancel scheduled recording(s)
//	This script actually produces no display.
//
//	Arguments:
//		tab=xyz		where cancellation was initiated
//		file=xyz	basename of file to be cancelled
//		choice=xyz	'cancel' or 'keep'
//		program=xyz	'skip' or 'cancelall' if regularly scheduled program
//
include('6ga-common.inc');

$bgcolor	= $TABCOLOR[$thistab];
$signature	= substr($thisfile, -8);

setcookie('tab', $thistab);

function killRecProcess($fbname) {
	global $VIDEOROOTDIR;

	$pid = @file_get_contents("$VIDEOROOTDIR/recorded/$fbname.stat");
	$pid = trim($pid);
	if (preg_match('/^\d+$/', $pid))
		system("kill $pid");
}

function futureEpisodes($fbn, $signature) {
	global $thisfile;
	return ("$fbn" >= "$thisfile") && preg_match("/-$signature\$/", $fbn);
}

function allEpisodes($fbn, $signature) {
	return preg_match("/-$signature\$/", $fbn);
}

//
// Cancel a scheduled recording
//
if ($choice == 'cancel') {
	$list = openList($thistab);
	readList($list, $thisfile);
	$date = $list['iepg']['month'] . '/' . $list['iepg']['date'];

	// Escape unescaped single quotes (apostrophes) saved in iEPG
	$alerttext	= sprintf("Recording of \"%s\" ",
				preg_replace("/\'/", "\\\'", $list['iepg']['program-title']));

  if (@$list['iepg']['program_id'] && @$list['iepg']['at-job']) {
	exec("/var/www/html/epgrec/sbin/dvr2-delresv.pl "
					. $list['iepg']['program_id']);
	closeList($list);
	$alerttext .= "has been cancelled ($date).";
  } else {
	switch($program) {
	case 'skip':
		$alerttext .= "will be skipped on $date.";

		//
		// Mark skipped so that daemon won't re-schedule this episode
		//
		rename($list['dirpath'] . "/$thisfile.iepg",
			   $list['dirpath'] . "/$thisfile.skip");
		killRecProcess($thisfile);
		break;

	case 'cancelall':
		$alerttext .= "has been cancelled for $date and beyond.";
		unlink($list['filepath']);

		//
		// Remove all or future episodes
		//
		$list = openList('scheduled',
						($thistab == 'scheduled')	? 'futureEpisodes'
													: 'allEpisodes',
						$signature);
		while (readList($list)) {
			unlink($list['dirpath'] . '/' . $list['fbasename'] . '.iepg');
			killRecProcess($list['fbasename']);
		}
		closeList($list);

		//
		// Find and remove the base regular schedule
		//
		$list = openList('regular', 'allEpisodes', $signature);
		while (readList($list)) {
			unlink($list['dirpath'] . '/' . $list['fbasename'] . '.iepg');
		}
		closeList($list);
		break;

	default:
		//
		// Single time recording - simply remove it.
		//
		$alerttext .= "has been cancelled ($date).";
		unlink($list['filepath']);
		killRecProcess($thisfile);

		//
		// Or if this is a regular scheduling, remove all single schedules
		// derived from this.
		//
		if ($thistab == 'regular') {
			$list = openList('scheduled', 'allEpisodes', $signature);
			while (readList($list)) {
				unlink($list['filepath']);
			}
			closeList($list);
		}
		break;
	}

	closeList($list);
	system("$SUDO $RCMDPATH/update-schedule -x >/dev/null 2>&1");
  }
}

printDocHeader();
$now	= time();

echo <<<END
<HTML>
<HEAD>
</HEAD>

<BODY BGCOLOR='$bgcolor'>
<SCRIPT LANGUAGE=JavaScript>
<!--
var choice		= '$choice';
var alerttext	= '$alerttext';

if (choice == 'cancel') {
	window.open('6ga.php?tab=$thistab&_time_=$now', '_6ga');
	alert(alerttext);
} else {
	window.open('6ga-panel.php?tab=$thistab&_time=$now', '_6ga_panel');
}
//-->
</SCRIPT>
</BODY>
</HTML>
END;
?>
