<?php
// $Id: client-index.php,v 1.31 2013/01/01 23:42:29 name Exp $
//
// index.php
//
//	This script manages 6ga recorded TV files.
//	Copyright (c) 2006-2013 Hiro Sugawara
//

if ($_SERVER['SERVER_PORT'] == '8000' &&
	(is_file('page.php') || is_link('page.php'))) {
	include('page.php');
	exit(0);
}


$encoding	= 'utf-8';

header('Content-type: text/html;charset=' . $encoding);
putenv('PATH=/usr/local/bin:' . getenv('PATH'));
$HTTPS = @$_SERVER['HTTPS'];

$AVIFNPAT	= '/^\\d{13}\\.(avi|mp4)/';
$SUBARCHPAT	= '/^A\\d{14}$/';

//
// As of php 5.0.4, many (if not all) file manipulation functions fail
// if the file size is no smaller than 2GB. Here are the alternatives.
//
function file_exists_2gb($filename) {
	system("test -f $filename", $rval);
	return ($rval == 0);
}

//
// If passed directory is a subarchive, return its title
//
function is_subarch($entry) {
	global $SUBARCHPAT;
	global $encoding;

	$encoded	= 'shift_jis';
	if (is_dir($entry) && preg_match($SUBARCHPAT, basename($entry))
		&& is_dir("$entry/comments")
		&& ($title = @file_get_contents("$entry/index.txt"))) {
		$title = trim($title);
		if (($fh = @fopen("$entry/index.iepg", 'r'))) {
			while (($line = fgets($fh))) {
				$line	= trim($line);
				if (preg_match('/charset=([^;]*)$/', $line, $matches)) {
					$encoded	= $matches[1];
					break;
				}
			}
			fclose($fh);
		}
		$title = trim($title);
		if (strcasecmp($encoded, $encoding)) {
			$title = iconv($encoded, $encoding, $title);
		}
		return $title;
	} else {
		return NULL;
	}
}

?>
<META HTTP_EQUIV='Refresh' CONTENT='600'>
<HTML>
<HEAD>
<TITLE>RECORDED TV PROGRAM FILE MANAGEMENT</TITLE>
<LINK REL='shortcut icon' HREF='favicon.ico'/>
<STYLE TYPE="text/css">
<!--
A {text-decoration:none}
A.anchor {
	color: #000000;
}
A:visited {
	color: #0000ff;
}
A:hover {
	color: #ff3f00;
	text-decoration: underline;
}
-->
</STYLE>
</HEAD>
<BODY>
<H2>Recorded TV Progam File Management</H2>
<?php
$self	= $_SERVER['PHP_SELF'];
$script	= $_SERVER['SCRIPT_NAME'];
//
// Table for current and jumpable locations
//
echo "<TABLE WIDTH=100% BORDER=0 CELLPADDING=0 CELLSPACING=0>\n";

//
// Determine where we are
//
$cwd = basename(getcwd());
$cpd = basename(dirname(getcwd()));

if ($cwd == 'archive')
	$MODE	= 'ARCHIVE';
else if ($cpd == 'archive')
	$MODE	= 'SUBARCHIVE';
else if ($cwd == '.deleted')
	$MODE	= 'DELETED';
else
	$MODE	= 'RECORDED';

if (!$HTTPS) {
	foreach (array_keys($_GET) as $gk)
		$_GET[$gk] = '';
	if ($MODE != 'RECORDED')
		exit(1);
}

//
// If archive, provide subarchive creation
//
if ($MODE == 'ARCHIVE') {
	echo <<<_END
<TR><TD COLSPAN=2 ALIGN=right>
<FORM ACTION=$self METHOD=GET><B>
Create a new subarchive
<INPUT NAME='newarchive' TYPE=text><INPUT TYPE=SUBMIT VALUE=Create>
</B></FORM></TD></TR>
_END;
}

$archname	= '';
if ($MODE == 'SUBARCHIVE') {
	$archname	= is_subarch("../$cwd");
}

//
// Print common header
//
$header	= array('ARCHIVE'		=> "Archived",
				'SUBARCHIVE'	=> "Archived \"$archname\"",
				'DELETED'		=> "Deleted",
				'RECORDED'		=> "Recorded");

// Determine free disk space
if (($fh = popen("df -H .|tail -1|awk '{print \$(NF-4), \$(NF-2)}'", 'r'))
		!= FALSE) {
	list($ts, $df) = split('[[:space:]]', trim(fgets($fh)));
	pclose($fh);
	if (substr($ts, -1) != 'B')
		$ts .= 'B';
	if (substr($df, -1) != 'B')
		$df .= 'B';
	$df = sprintf(" (free space: %s/%s)", $df, $ts);
}
printf("<TR><TD><FONT SIZE='+1'><B>%s files directory%s</B></FONT></TD>\n",
		$header[$MODE], $df);


//
// Convert file name to date string
//
function fn2date($file) {
	return sprintf("%04d/%2d/%2d",
					substr($file, 0, 4),
					substr($file, 4, 2),
					substr($file, 6, 2));
}

//
// Save subarchive list
//
function savearchs($list) {
	if (($fh = fopen('.subarchives', 'w')) != FALSE) {
		foreach ($list as $arch) {
			fputs($fh, "$arch\n");
		}
		fclose($fh);
	}
}

//
// Create subarchive list
//
function subarchs() {
	global $SUBARCHPAT;

	$sortlist	= array();
	if (($fh = @fopen('.subarchives', 'r')) != FALSE) {
		while ($arch = fgets($fh)) {
			$arch	= trim($arch);
			if (preg_match($SUBARCHPAT, $arch)) {
				array_push($sortlist, $arch);
			}
		}
		fclose($fh);
	}

	$list	= array();
	if (($dh = opendir('.')) != FALSE) {
		while ($entry = readdir($dh)) {
			if (is_subarch($entry)) {
				array_push($list, $entry);
			}
		}
		closedir($dh);
	}
	sort($list);
	if (count($sortlist) == count($list))
		return $sortlist;

	savearchs($list);
	return $list;
}

//
// Get list of subarchives
//
if ($MODE == 'ARCHIVE') {
	$archs	= subarchs();

	//
	// Create a new sub-archive?
	//
	if (trim($newarchive = @$_GET['newarchive']) != ''
		&& substr($newarchive, 0, 1) != '.') {
		$match = 0;
		if (($dh = opendir('.')) != NULL) {
			while ($entry = readdir($dh)) {
				if ($newarchive == is_subarch($entry)) {
					$match++;
					break;
				}
			}
			closedir($dh);
		}
		if (!$match) {
			$dir = sprintf("%s/A%s", getcwd(), strftime("%Y%m%d%H%M%S"));
			if (@mkdir($dir)) {
				mkdir("$dir/comments");
				symlink('../index.php', "$dir/index.php");
				if (($fh = fopen($dir . '/index.txt', 'w')) != NULL) {
					fputs($fh, "$newarchive\n");
					fclose($fh);
				}
				if (($fh = fopen($dir . '/index.iepg', 'w')) != NULL) {
					fputs($fh,
"Content-type: application/x-tv-program-info; charset=$encoding\n");
					fclose($fh);
				}
				array_push($archs, basename($dir));
				savearchs($archs);
			}
		}
	}

	//
	// Sort subarchives?
	//
	if (($sort = @$_GET['sort']) != '' && ($file = $_GET['file']) != '') {
		for ($inx = 0; $inx < count($archs); $inx++) {
			if ($archs[$inx] == $file) {
				break;
			}
		}
		if ($inx < count($archs)) {
			array_splice($archs, $inx, 1);
			if ($sort	== 'top') {
				array_unshift($archs, $file);
			} else if ($sort	== 'up') {
				array_splice($archs, $inx - 1, 0, $file);
			} else if ($sort	== 'down') {
				array_splice($archs, $inx + 1, 0, $file);
			} else if ($sort	== 'bottom') {
				array_push($archs, $file);
			}
		}
		
		savearchs($archs);
	}

	//
	// Delete a subarchive?
	//
	if (($deldir = @$_GET['delsubarch']) != '' && subarchempty($deldir)) {
		if (rmdir("$deldir/comments")
			&& unlink("$deldir/index.php")
			&& unlink("$deldir/index.txt")
			&& unlink("$deldir/index.iepg")
			&& rmdir("$deldir")) {
			for ($inx = 0; $inx < count($archs); $inx++) {
				if ($archs[$inx] == $deldir) {
					break;
				}
			}
			if ($inx < count($archs)) {
				array_splice($archs, $inx, 1);
				savearchs($archs);
			}
		}
	}
}

$goto	= array('ARCHIVE'		=> "<A HREF='../'>Recorded</A>",
				'SUBARCHIVE'	=> "<A HREF='../'>Archived</A>",
				'DELETED'		=> "<A HREF='../'>Recorded</A>",
				'RECORDED'		=> "<A HREF='archive/'>Archived</A>" .
									" or <A HREF='.deleted/'>Deleted</A>");

if ($HTTPS)
	printf("<TD ALIGN=RIGHT>Go to %s files directory</TD>\n", $goto[$MODE]);
echo "</TR></TABLE>\n";

function move2dir($fbname, $dir) {
	rename("$fbname.avi", "$dir/$fbname.avi");
	rename("$fbname.mp4", "$dir/$fbname.mp4");
	rename("comments/$fbname.txt", "$dir/comments/$fbname.txt");
	rename("comments/$fbname.iepg", "$dir/comments/$fbname.iepg");
}

$sortby	= '';
if (($sortby = @$_GET['sortby'])) {
	$sortby	= substr($sortby, 0, 1);
}

if (($archfile = @$_GET['arch'])) {
	if (is_dir('archive')) {
		move2dir($archfile, 'archive');
	}
} else if (($delfile = @$_GET['del'])) {
	if (is_dir('.deleted')) {
		move2dir($delfile, '.deleted');
	} else if (is_dir('../.deleted') && $cwd != '.deleted') {
		move2dir($delfile, '../.deleted');
	} else {
		@unlink("$delfile.avi");
		@unlink("$delfile.mp4");
		@unlink("comments/$delfile.txt");
		@unlink("comments/$delfile.iepg");
	}
} else if (($resfile = @$_GET['res'])) {
	if (!is_dir('archive') && !is_dir('.deleted')) {
		move2dir($resfile, '..');
	}
} else if (($sendto = @$_GET['sendto']) && ($file = @$_GET['file'])) {
	if (is_dir($sendto)) {
		move2dir($file, $sendto);
	}
}

//
// Main table for recorded files
//
echo "<TABLE WIDTH=100%>\n";

$BGCOLORH	= '#9090ff';
$BGCOLOR	= array('#e0e0ff', '#f7f7f7');
$bgcolor	= 0;

$tsort	= ($sortby == 't') ? 'T' : 't';
$dsort	= ($sortby == 'd') ? 'D' : 'd';

echo <<<_END
<TR BGCOLOR='$BGCOLORH'>
	<TH><A HREF='$script?sortby=$tsort'>Date/Time</A></TH>
	<TH><A HREF='$script?sortby=$dsort'>Description</A></TH>
	<TH>Type</TH>
	<TH>Length</TH>
	<TH>File</TH>
</TR>
_END;

//
// Return programs in a subarchive
//
function insubarch($arch) {
	global	$AVIFNPAT;

	$list	= array();
	if (($dh = opendir($arch)) != FALSE) {
		while ($entry = readdir($dh)) {
			if (preg_match($AVIFNPAT, $entry)) {
				array_push($list, $entry);
			}
		}
		closedir($dh);
	}
	sort($list);
	return $list;
}

//
// Check if a subarchive is empty - RACY!
//
function subarchempty($arch) {
	return is_dir($arch) && (count(insubarch($arch)) == 0);
}

//
// Scan for subarchives
//
if (@$archs) {
	$pos	= 0;
	$count	= count($archs);
	foreach ($archs as $arch) {
		if (($title	= is_subarch($arch)) != NULL) {
			printf("<TR BGCOLOR='%s'><FORM METHOD=GET ACTION='$self'>\n",
					$BGCOLOR[$bgcolor]);
			$list	= insubarch($arch);
			if (count($list) == 0)
				printf("<TD></TD>\n");
			else if (count($list) == 1)
				printf("<TD>%s</TD>\n", fn2date($list[0]));
			else
				printf("<TD><TABLE WIDTH=100%%><TR><TD>%s</TD></TR>" .
						"<TR><TD ALIGN=RIGHT>- %s</TD></TR></TABLE></TD>\n",
										fn2date($list[0]),
										fn2date($list[count($list) - 1]));
			printf("<TD><A HREF='%s/'>%s</A></TD>\n", $arch, $title);
			printf("<TD></TD>\n");
			printf("<TD ALIGN=RIGHT>%d&nbsp;episodes</TD>\n", count($list));
			echo("<TD>");
			if (subarchempty($arch))
				echo("<A HREF='$script?delsubarch=$arch&sortby=$sortby'>Delete</A><BR>");
			echo("<SELECT NAME='sort'>\n");
			if ($count > 1) {
				echo("<OPTION>- Sort subarchives -\n");
				if ($pos) {
					echo("<OPTION VALUE='top'>Move to top\n");
					echo("<OPTION VALUE='up'>Move up\n");
				}
				if ($pos < $count - 1) {
					echo("<OPTION VALUE='down'>Move down\n");
					echo("<OPTION VALUE='bottom'>Move to bottom\n");
				}
			}
			echo("</SELECT>\n");
			echo "<INPUT TYPE=SUBMIT VALUE='Sort'>\n";
			echo "<INPUT TYPE=HIDDEN NAME='file' VALUE='$arch'>\n";
			echo("</TD></FORM></TR>\n");
			$subtitles[$arch]	= $title;
			$bgcolor++;
			$bgcolor %= 2;
			$pos++;
		}
	}
}

//
// Scan for avi files
//
$files	= array();
$dh = opendir('.');
while (($file = readdir($dh)) != FALSE) {
	if (preg_match($AVIFNPAT, $file)) {
		$files[]	= substr($file, 0, 13);
	}
}
closedir($dh);

//
// Sort by description
//
function cmpdesc($a, $b) {
	$A	= file_get_contents("comments/$a.txt");
	$B	= file_get_contents("comments/$b.txt");
	return strcmp($A, $B);
}

//
// Sort by 2nd part of description
//
function cmpdesc2($a, $b) {
	$A	= file_get_contents("comments/$a.txt");
	$AA	= split('/', $A);
	$AAA = array_pop($AA);
	$B	= file_get_contents("comments/$b.txt");
	$BB	= split('/', $B);
	$BBB = array_pop($BB);
	return strcmp($AAA, $BBB);
}

if (!$sortby || $sortby	== 't') {
	sort($files);
} else if ($sortby	== 'T') {
	rsort($files);
} else if ($sortby	== 'd') {
	usort($files, 'cmpdesc');
} else if ($sortby	== 'D') {
	usort($files, 'cmpdesc2');
}

$prevtitle='';

foreach ($files as $file) {
	printf("<TR BGCOLOR='%s'>\n", $BGCOLOR[$bgcolor]);
	echo "<FORM METHOD=GET ACTION='$self'>\n";
	$cfile = "comments/$file.txt";
	$ifile = "comments/$file.iepg";
	$avifile	= file_exists_2gb("$file.mp4") ? "$file.mp4" : "$file.avi";

	// Date and time
	printf("<TD NOWRAP>%s&nbsp;%2d:%02d</TD>\n",
				fn2date($file),
				substr($file, 8, 2),
				substr($file,10, 2));

	// Title and size
	$progtitle	= '';
	$length	= '';
	$ftype	= '';
	$start	= NULL;
	$end	= NULL;
	if (($fh = @fopen($ifile, 'r')) != NULL) {
		$encoded	= 'shift_jis';
		$vcodec	= '';
		while (($line = @fgets($fh)) != NULL) {
			$line = trim($line);
			if (preg_match('/^start-recording: /', $line)) {
				$start = preg_split('/\s/', $line);
				$start = $start[1];
			} else if (preg_match('/^end-recording: /', $line)) {
				$end = preg_split('/\s/', $line);
				$end = $end[1];
			} else if (preg_match('/charset=([^;]*)$/', $line, $matches)) {
				$encoded	= $matches[1];
			} else if (preg_match('/vsize: (.*)$/', $line, $matches)) {
				$ftype	= $matches[1];
			} else if (preg_match('/vcodec: (.*)$/', $line, $matches)) {
				$vcodec	= $matches[1];
			}
		}
		fclose($fh);
		if ($ftype && $vcodec) {
			$ftype .= "<BR>$vcodec";
		}

		$progtitle	= file_get_contents($cfile);
		if (strcasecmp($encoded, $encoding)) {
			$progtitle	= iconv($encoded, $encoding, $progtitle);
		}
		if (@$start && @$end) {
			$start	= preg_split('/:/', $start);
			$end	= preg_split('/:/', $end);
			$hour	= $end[0] - $start[0];
			$min	= $end[1] - $start[1];
			if ($min < 0) {
				$min	+= 60;
				$hour--;
			}
			if ($hour < 0) {
				$hour +=24;
			}
			$avifsize	= exec(
	"ls -l --block-size=1000000 $avifile|awk '{print $5}'");
			$length	= sprintf("<A HREF='%s'>%dh%02dm</A><BR>%dMB",
					$avifile, $hour, $min, $avifsize);
		}

		if (!$ftype) {
			$ftas	= array();
			$ftype	= exec("exiftool -T -imagesize -videocodec $avifile");
			$ftype	= preg_replace('/\s+/', '<BR>', $ftype);
		}
	}

	if ($prevtitle == $progtitle) {
		echo("<TD BGCOLOR='ffff00'>$progtitle</TD>\n");
	} else {
		echo("<TD>$progtitle</TD>\n");
	}
	$prevtitle = $progtitle;
	echo("<TD ALIGN='right'>$ftype</TD>\n");
	echo("<TD ALIGN='right'>$length</TD>\n");

	echo("<TD NOWRAP>\n");
	if ($MODE != 'SUBARCHIVE' && $HTTPS)
		echo("<A HREF='$script?del=$file&sortby=$sortby'>Delete</A>");
	if ($MODE == 'ARCHIVE')
		echo(" ");
	else if ($HTTPS)
		echo("<BR>");
	else
		echo("Function<BR>disabled");
	if ($MODE == 'RECORDED' && $HTTPS)
		echo("<A HREF='$script?arch=$file'>Archive</A>\n");
	else if ($HTTPS)
		echo("<A HREF='$script?res=$file'>Restore</A>\n");
	if ($MODE == 'ARCHIVE') {
		echo "<BR><SELECT NAME='sendto'>\n";
		echo "<OPTION>- Select subarchive -\n";
		foreach ($archs as $arch) {
			if (!($subtitle	= @$subtitles[$arch]))
				continue;
			if (mb_strlen($subtitle, $encoding) > 10)
				$subtitle	= mb_substr($subtitle, 0, 10, $encoding) . '...';
			printf("<OPTION VALUE='%s'>%s\n", $arch, $subtitle);
		}
		echo "</SELECT>\n";
		echo "<INPUT TYPE=SUBMIT VALUE='Save'>\n";
		echo "<INPUT TYPE=HIDDEN NAME='file' VALUE='$file'>\n";
	}
	echo("</TD>\n");

	echo "</FORM>\n";
	echo("</TR>\n");
	$bgcolor++;
	$bgcolor %= 2;
}
?>
</TABLE>
</BODY>
</HTML>
